const {
  app, BrowserWindow, shell, 
} = require('electron');
const path = require('path');
const { appIcon, appUrl } = require('./constants/other')
const MainHelper = require('./utils/helpers/main.helper')
const StartUpApp = require('./src/app/startup');
const { windowsConfig, appName } = require('./constants/core.js');

let mainWindow = null;

const createWindow = () => {
  if (mainWindow) return mainWindow.focus();
  mainWindow = new BrowserWindow({
    width: 1040,
    height: 800,
    minWidth: 400,
    minHeight: 700, 
    show: true,
    icon: path.join(__dirname, appIcon),
    webPreferences: {
      preload: path.join(__dirname, 'preload.js'),
      nodeIntegration: false,
      contextIsolation: true,
    },
  });
     
  mainWindow.loadURL(appUrl);
  mainWindow.setTitle(appName)
  mainWindow.webContents.on('did-finish-load', () => console.log(`1. Current URL: ${mainWindow.webContents.getURL()}`));
  mainWindow.setAutoHideMenuBar(true);
  mainWindow.setMenuBarVisibility(false);
  MainHelper.setAppVersionToLocalStorage(mainWindow);
  mainWindow.webContents.on('context-menu', (_, props) => MainHelper.copyPastLogic(props));
  mainWindow.webContents.on('did-create-window', (newWindow, event) => {
    const { url } = event;
    newWindow.close()
    if (!url.includes(appUrl)) return shell.openExternal(url);
    mainWindow.show();
    mainWindow.focus()
  });
  mainWindow.on('closed', () => console.log('Bye'));
};

const middlewares = () => {
  StartUpApp.run();
  app.setAppUserModelId(windowsConfig.appName);
  app.disableHardwareAcceleration();
}

const openWindow = () => {
  const gotSingleInstanceLock = app.requestSingleInstanceLock();
  if (!gotSingleInstanceLock) return app.quit();
  app.on('second-instance', () => {
    if (mainWindow) {
      if (mainWindow.isMinimized()) mainWindow.restore();
      mainWindow.focus();
    }
  });
  app.on('ready', createWindow);
  app.on('window-all-closed', () => {
    app.quit();
  });
  app.on('activate', () => mainWindow && createWindow());
}

const init = () => {
  middlewares()
  openWindow()
}

(()=>{
  try {
    init()
  } catch (error) {
    console.log(error);
  }
})()